#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const literator_1 = require("./literator");
const path_1 = require("path");
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const config_1 = require("./config");
const package_json = fs_extra_1.readJSONSync(path_1.resolve(__dirname, "../../package.json"));
program.version(package_json.version);
program.option("-w, --watch", "compose file or files in directory in watch mode");
program.option("-clean, --clean", "remove generated files");
program.parse(process.argv);
const args = program.args;
if (args.length === 0) {
    program.help();
}
else {
    const [target] = args;
    if (target) {
        if (program.clean) {
            literator_1.CrossoutDirectory(target);
        }
        else {
            Transcribe(target);
        }
    }
}
function Transcribe(target) {
    //
    const working_directory = process.cwd();
    const project_package = fs_extra_1.readJsonSync(path_1.join(working_directory, "package.json"), { throws: false }) || { devDependencies: {} };
    //
    const dsl_names = Object.keys(project_package.devDependencies).filter(key => key.startsWith("draft-dsl"));
    const dsls = dsl_names.map(name => { var _a; return (_a = require(`${path_1.join(working_directory, "node_modules", name)}`)) === null || _a === void 0 ? void 0 : _a.MakeDSL(); });
    config_1.config.dsls = dsls;
    //
    const path = path_1.resolve(working_directory, target);
    if (fs_1.lstatSync(path).isDirectory()) {
        program.watch ? literator_1.InspectDirectory(path) : literator_1.ComposeDirectory(path);
    }
    else {
        program.watch ? literator_1.InspectFile(path) : literator_1.ComposeFile(path);
    }
}
